from django.conf import settings
from django.utils.translation import gettext_lazy as _

from django_daisy._helpers import ASSET_PATH

# Define defaults for DAISY_SETTINGS
DEFAULT_DAISY_SETTINGS = {
    "SITE_TITLE": "Django Admin",
    "SITE_HEADER": "Administration",
    "INDEX_TITLE": "hi, welcome to your dashboard",
    "SITE_LOGO": f"{ASSET_PATH}admin/img/daisyui-logomark.svg",
    "EXTRA_STYLES": [],
    "EXTRA_SCRIPTS": [],
    "LOAD_FULL_STYLES": False,
    "SHOW_CHANGELIST_FILTER": False,
    "DONT_SUPPORT_ME": False,
    "SIDEBAR_FOOTNOTE": "",
    "FORM_RENDERER": "django.forms.renderers.TemplatesSetting",
    "X_FRAME_OPTIONS": "SAMEORIGIN",
    "DEFAULT_THEME": 'None',
    "DEFAULT_THEME_DARK": None,
    "SHOW_THEME_SELECTOR": True,
    "THEME_LIST": [
        {"name": _("Light"), "value": "light"},
        {"name": _("Dark"), "value": "dark"},
        {"name": _("CMYK"), "value": "cmyk"},
        {"name": _("Dracula"), "value": "dracula"},
        {"name": _("Lemonade"), "value": "lemonade"},
        {"name": _("Halloween"), "value": "halloween"},
        {"name": _("Garden"), "value": "garden"},
    ],
    "APPS_REORDER": {
        "auth": {
            "icon": "fa-solid fa-person-military-pointing",
            "name": _("Authentication"),
            "hide": False,
            "app": "users",
            # 'priority': 1,  # higher value will appear on top items
        },
    },
}

# Get DAISY_SETTINGS from settings.py or fall back to defaults
DAISY_SETTINGS = getattr(settings, "DAISY_SETTINGS", DEFAULT_DAISY_SETTINGS)

# Ensure any missing keys from defaults are included
for key, value in DEFAULT_DAISY_SETTINGS.items():
    DAISY_SETTINGS.setdefault(key, value)

settings.FORM_RENDERER = DAISY_SETTINGS["FORM_RENDERER"]

settings.X_FRAME_OPTIONS = DAISY_SETTINGS["X_FRAME_OPTIONS"]



import logging

from django.apps import apps
from django.conf import settings
from django.contrib import admin
from django.core.files.storage import FileSystemStorage
from django.http import HttpResponse
from django.shortcuts import render
from django.urls import path, reverse
from django.views.decorators.csrf import csrf_exempt

from django_daisy._helpers import ASSET_PATH
from django_daisy.module_settings import DAISY_SETTINGS

logger = logging.getLogger(__name__)


# Remove default form fields for specific date and time fields
# admin.options.FORMFIELD_FOR_DBFIELD_DEFAULTS.pop(models.DateTimeField, None)
# admin.options.FORMFIELD_FOR_DBFIELD_DEFAULTS.pop(models.DateField, None)
# admin.options.FORMFIELD_FOR_DBFIELD_DEFAULTS.pop(models.TimeField, None)


class DaisyAdminSite(admin.AdminSite):
    password_change_template = "admin/registration/password_change_form.html"
    password_change_done_template = "admin/registration/password_change_done.html"
    site_title = DAISY_SETTINGS.get("SITE_TITLE", "django admin")
    site_header = DAISY_SETTINGS.get("SITE_HEADER", "Administration")
    index_title = DAISY_SETTINGS.get("SITE_HEADER", "hi, welcome to your dashboard")
    logo = DAISY_SETTINGS.get(
        "SITE_LOGO", f"{ASSET_PATH}admin/img/daisyui-logomark.svg"
    )

    def get_urls(self):
        urls = [
            path("json-editor-upload-handler/", self.admin_view(self.upload_file), name='json-editor-upload-handler')
        ]
        return urls + super().get_urls()

    def get_log_entries(self, request):
        from django.contrib.admin.models import LogEntry

        return LogEntry.objects.select_related("content_type", "user")

    def index(self, request, extra_context=None):
        """
        Display the main admin index page, which lists all of the installed
        apps that have been registered in this site.
        """

        logentry_changelist_url = reverse("admin:admin_logentry_changelist")

        app_list = self.get_app_list(request)

        context = {
            **self.each_context(request),
            "latest_history": self.get_log_entries(request)[:15],
            "title": self.index_title,
            "app_list": app_list,
            "logentry_changelist_url": logentry_changelist_url,
            **(extra_context or {}),
        }

        request.current_app = self.name
        return render(request, self.index_template or "admin/index.html", context)

    # Overriding the get_app_list method to sort apps and exclude hidden ones
    def get_app_list(self, request, app_label=None):
        """
        Return a sorted list of all installed apps that are registered in this site,
        excluding hidden apps and adding optional icons and dividers.
        """
        app_dict = self._build_app_dict(request, app_label)
        sorted_app_list = sorted(
            app_dict.values(), key=lambda x: x.get("priority", 0), reverse=True
        )
        return [app for app in sorted_app_list if not app.get("hide", False)]

    def _build_app_dict(self, request, label=None):
        """
        Builds and modifies the app dictionary to include icons and app grouping logic.
        """
        app_dict = super()._build_app_dict(request, label)

        if label:
            return app_dict

        modified_app_dict = app_dict.copy()
        override_apps_config = DAISY_SETTINGS.get("APPS_REORDER", {})

        for app_label, app_info in app_dict.items():
            # Add icon and divider title to each app
            app_info["icon"] = getattr(apps.get_app_config(app_label), "icon", "")
            app_info["divider_title"] = getattr(
                apps.get_app_config(app_label), "divider_title", ""
            )

            if app_label in override_apps_config:
                app_info.update(override_apps_config[app_label])

        return modified_app_dict

    def each_context(self, request):
        context = super().each_context(request)
        try:
            change_language_url = reverse("set_language")
        except:
            change_language_url = None

        return {
            **context,
            **DAISY_SETTINGS,
            "change_language_url": change_language_url,
            "logo": self.get_logo(request),
            "can_delete_popup": "",
            "use_i18n": getattr(settings, "USE_I18N", False),
        }

    def get_logo(self, request):
        return self.logo

    @csrf_exempt
    def upload_file(self, request):
        if request.method != "POST":
            return HttpResponse(
                "Invalid request method", status=405, content_type="text/plain"
            )

        if "file" not in request.FILES:
            return HttpResponse(
                "No file uploaded", status=400, content_type="text/plain"
            )

        file = request.FILES["file"]

        try:
            fs = FileSystemStorage()
            filename = fs.save(file.name, file)
            url = request.build_absolute_uri(fs.url(filename))
            return HttpResponse(url, content_type="text/plain")

        except Exception as e:
            # Log the error for debugging purposes
            logger.error("Error saving file: %s", e)
            # Return a generic error message to the client
            return HttpResponse(
                "Failed to save file", status=500, content_type="text/plain"
            )



from django.db import models
from django import forms
from django.forms.widgets import Textarea


class JsonKeyValueWidget(Textarea):
    template_name = 'django/forms/widgets/keyvalue_field.html'

    def __init__(self, attrs={}):
        attrs['class'] = 'keyvalue-json-editor-field'
        super().__init__(attrs)

    class Media:
        js = (
            'admin/fields/keyvalue_field/script.js',
        )
        css = {
            'all': (
                'admin/fields/keyvalue_field/style.css',
            )
        }


class JsonKeyValueField(models.JSONField):
    description = "custom json key value field"

    def formfield(self, **kwargs):
        return super().formfield(**{
            'widget': JsonKeyValueWidget,
            'encoder': self.encoder,
            'decoder': self.decoder,
            **kwargs,
        })



from django.apps import AppConfig
from django.conf import settings
from django.contrib import admin
from django.contrib.admin import sites
from django.utils.module_loading import import_string

from django_daisy import module_settings  # noqa, load module settings

if default_admin_site := getattr(settings, 'DEFAULT_ADMIN_SITE_CLASS', None):
    AdminSiteClass = import_string(default_admin_site)
else:
    AdminSiteClass = import_string('django_daisy.admin.DaisyAdminSite')


class DefaultAppConfig(AppConfig):
    name = "django_daisy"
    default = True

    def ready(self):
        site = AdminSiteClass()
        from .logentry_admin import LogentryAdmin  # noqate
        from django.contrib.admin.models import LogEntry  # noqate

        admin.site = site
        sites.site = site

        admin.site.register(LogEntry, LogentryAdmin)


class BasicAppConfig(AppConfig):
    name = "django_daisy"



import re
from typing import Optional, Dict, Any
from urllib.parse import parse_qs

from django import template
from django.contrib.admin.models import LogEntry
from django.contrib.admin.templatetags import admin_list
from django.contrib.admin.templatetags.admin_list import result_list, result_headers, result_hidden_fields, results
from django.contrib.admin.templatetags.base import InclusionAdminNode
from django.contrib.admin.views.main import PAGE_VAR
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.http import HttpRequest
from django.template.loader import get_template
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe, SafeString

register = template.Library()

# Constants
BOOLEAN_ICON_CLASSES = {
    True: "text-base fa fa-circle-check text-success",
    False: "text-base fa fa-circle-xmark text-error",
    None: "text-base fa fa-question-circle",
}

MULTIPLE_SELECT_FILTERS = [
    'ChoicesFieldListFilter',
    'AllValuesFieldListFilter',
    'RelatedOnlyFieldListFilter',
    'RelatedFieldListFilter',
]


def custom_boolean_icon(field_val: Optional[bool]) -> SafeString:
    """Custom boolean icon for Django admin with improved styling."""
    icon_class = BOOLEAN_ICON_CLASSES.get(field_val, BOOLEAN_ICON_CLASSES[None])
    return format_html('<i class="{}"></i>', icon_class)


# Monkey patch the admin boolean icon
admin_list._boolean_icon = custom_boolean_icon


@register.simple_tag
def get_recent_changes(model_instance):
    """
    Template tag to retrieve the 4 most recent LogEntry changes for a specific model instance.

    Usage in template:
    {% get_recent_changes object %} - gets 4 most recent changes for the object
    """
    content_type = ContentType.objects.get_for_model(model_instance)

    return LogEntry.objects.select_related('user', 'content_type').filter(
        content_type=content_type,
        object_id=str(model_instance.pk)
    ).order_by('-action_time')[:4]


def daisy_result_list(cl):
    """
    Display the headers and data list together.
    """
    headers = list(result_headers(cl))
    num_sorted_fields = 0
    for h in headers:
        if h["sortable"] and h["sorted"]:
            num_sorted_fields += 1

    return {
        "cl": cl,
        "result_hidden_fields": list(result_hidden_fields(cl)),
        "result_headers": headers,
        "num_sorted_fields": num_sorted_fields,
        "results": list(results(cl)),
    }


@register.tag(name="daisy_result_list")
def daisy_result_list_tag(parser, token):
    return InclusionAdminNode(
        parser,
        token,
        func=daisy_result_list,
        template_name="change_list_results.html",
        takes_context=False,
    )


# String manipulation filters
@register.filter(name='replace')
def replace_filter(value: str, args: str) -> str:
    """Replace substring in value with new substring."""
    try:
        old_val, new_val = args.split(',', 1)
        return str(value).replace(old_val, new_val)
    except ValueError:
        return str(value)


@register.filter(name='add_class')
def add_class_filter(value: Any, css_class: str) -> Any:
    """Add CSS class to form widget or HTML string."""
    if hasattr(value, 'as_widget'):
        return value.as_widget(attrs={'class': css_class})
    elif isinstance(value, str):
        return _append_class_to_html_tag(value, css_class)
    return value


@register.filter(name='apply_class_to_widget')
def apply_class_to_widget_filter(widget: Dict[str, Any], new_classes: str) -> Dict[str, Any]:
    """Apply additional CSS classes to a widget dictionary."""
    current_class = widget.get('attrs', {}).get('class', '')
    widget.setdefault('attrs', {})['class'] = f"{current_class} {new_classes}".strip()
    return widget


@register.filter
def sum_errors(formset) -> int:
    """Calculate total number of errors in a formset."""
    field_errors = sum(len(form.errors) for form in formset)
    non_form_errors = (
        len(formset.non_form_errors())
        if hasattr(formset, 'non_form_errors')
        else 0
    )
    return field_errors + non_form_errors


# Simple tags for requests and popups
@register.simple_tag(name="popup_val")
def popup_value_tag(request: HttpRequest) -> Optional[str]:
    """Get popup parameter from request."""
    return request.GET.get("_popup")


@register.simple_tag
def get_page_link(cl, page_num: int) -> str:
    """Generate pagination link for Django admin changelist."""
    return cl.get_query_string({PAGE_VAR: page_num})


@register.simple_tag
def get_user_admin_change_url(user) -> str:
    """Generate admin change URL for a user instance."""
    try:
        User = get_user_model()
        app_label = User._meta.app_label
        model_name = User._meta.model_name
        return reverse(f"admin:{app_label}_{model_name}_change", args=[user.pk])
    except Exception:
        return '#'


@register.simple_tag
def get_app_icon(*apps) -> str:
    """Get icon attribute from the first app that has one."""
    for app in apps:
        if hasattr(app, 'icon'):
            return app.icon
    return ''


# Admin list filter tags
@register.simple_tag
def admin_list_filter(cl, spec, request: HttpRequest) -> SafeString:
    """Render admin list filter template."""
    template = get_template(spec.template)
    context = {
        "title": spec.title,
        "choices": list(spec.choices(cl)),
        "spec": spec,
    }
    return template.render(context, request=request)


@register.simple_tag
def get_filter_keys(spec) -> str:
    """Get comma-separated list of filter parameter keys."""
    return ",".join(spec.expected_parameters())


@register.simple_tag
def get_active_filters_count(cl) -> int:
    """Get count of active filters in changelist."""
    return len(cl.filter_params) if hasattr(cl, 'filter_params') else 0


@register.simple_tag
def is_multiple_filter_choice(spec) -> str:
    """Check if filter spec supports multiple selections."""
    from django.contrib.admin import filters

    multiple_select_classes = [
        filters.ChoicesFieldListFilter,
        filters.AllValuesFieldListFilter,
        filters.RelatedOnlyFieldListFilter,
        filters.RelatedFieldListFilter,
    ]

    if (spec.__class__ in multiple_select_classes or
            getattr(spec, 'multiple', False)):
        return 'multiple'
    return ''


@register.simple_tag
def is_active_choice(choice: Dict[str, Any], spec, request: HttpRequest) -> str:
    """Check if a filter choice is currently active/selected."""
    if choice.get('selected'):
        return 'selected'

    for filter_key in spec.expected_parameters():
        if "expertise" in filter_key:
            print('x')
        if is_multiple_filter_choice(spec):
            filter_key = filter_key.replace('__exact', '__in')
            if not filter_key.endswith('__in'):
                filter_key = filter_key + '__in'

        choice_value = get_bare_option_value(spec, choice)
        current_values = request.GET.get(filter_key)

        if current_values and choice_value:
            if choice_value in current_values.split(','):
                return 'selected'

    return ''


@register.simple_tag
def get_choice_value(spec, choice: Dict[str, Any]) -> str:
    """Get the query string value for a filter choice."""
    query_string = choice['query_string']
    # Future enhancement: handle multiple filter logic here if needed
    return query_string


@register.simple_tag
def get_bare_option_value(spec, choice: Dict[str, Any]) -> str:
    """Extract bare option values from choice query string."""
    values = []
    query_string = choice['query_string']

    if query_string.startswith('?'):
        query_string = query_string[1:]

    for filter_key in spec.expected_parameters():
        parsed_values = parse_qs(query_string).get(filter_key)
        if parsed_values and parsed_values[0]:
            values.append(parsed_values[0])

    return ','.join(values) if values else ''


# Private helper functions
def _append_class_to_html_tag(html_tag: str, new_class: str) -> SafeString:
    """Append CSS class to HTML tag string."""
    class_pattern = r'(class="[^"]*)(")'

    if re.search(class_pattern, html_tag):
        # Append to existing class attribute
        html_tag = re.sub(class_pattern, rf'\1 {new_class}\2', html_tag)
    else:
        # Add new class attribute
        html_tag = re.sub(r'(<[^>]*)(>)', rf'\1 class="{new_class}"\2', html_tag)

    return mark_safe(html_tag)
